@@ -1,9 +1,11 @@
 from hydenv.database import HydenvDatabase
 from hydenv.measurements import HydenvMeasurementsCli
+from hydenv.exercises import HydenvExercises
 
 
 import fire
 fire.Fire({
     'database': HydenvDatabase,
-    'measurements': HydenvMeasurementsCli
+    'measurements': HydenvMeasurementsCli,
+    'exercises': HydenvExercises
 })
